// $Id: clnt.h,v 1.2 1998/10/20 02:33:20 levine Exp $
// -*- C++ -*-
// ============================================================================
//
// = LIBRARY
//    TAO/tests/Cubit/CORBAplus/IDL_Cubit
//
// = FILENAME
//    clnt.h
//
// = DESCRIPTION
//      This class implements a simple "cube" CORBA client for the Cubit
//      example using stubs generated by the TAO ORB IDL compiler.
//
// = AUTHORS
//      Andy Gokhale, Sumedh Mungee, and Sergio Flores-Gaitan
//
// ============================================================================
#ifndef _CLNT_H_
#define _CLNT_H_

#include "ace/Get_Opt.h"

#if !defined (ACE_LACKS_PRAGMA_ONCE)
# pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#include "pbroker/corba/orb.h"
#include "pbroker/corba/request.h"
#include "pbroker/corba/environ.h"
#include "cubit.h"

class Cubit_Client
  // = TITLE
  //    Defines a class that encapsulates behaviour of the Cubit client
  //    example.  Provides a better understanding of the logic in an
  //    object oriented way.
  //
  // = DESCRIPTION
  //    This class declares an interface to run the example client for
  //    Cubit CORBA server.  All the complexity for initializing the
  //    server is hidden in the class.  Just the run() interface is needed.
{
 public:
  // = Constructor and destructor.
  Cubit_Client (void);
  ~Cubit_Client (void);

  int run (void);
  // Execute client example code.

  int init (int argc, char **argv);
  // Initialize the client communication endpoint with server.

private:
  int func (u_int i);
  // simple function that returns the substraction of 117 from the parameter.

  int parse_args (void);
  // Parses the arguments passed on the command line.

  void print_stats (const char *call_name, ACE_Profile_Timer::ACE_Elapsed_Time &elapsed_time);
  // prints the time stats

  void cube_union_stub (void);
  // Exercise the union.  Cube a union.

  void cube_union_dii (void);
  // Exercise the union using dii.

  void cube_struct (int i);
  // calculate the cube from a struct.

  void cube_long (int i);
  // calculate the cube from a long

  void cube_octet (int i);
  // Cube an octet

  void cube_short (int i);
  // Cube a short.

  void cube_struct_dii (void);
  // Cube the numbers in a struct

  int argc_;
  // # of arguments on the command line.

  char **argv_;
  // arguments from command line.

  const char *cubit_factory_key_;
  // Key of factory obj ref.

  char *cubit_key_;
  // Key of the obj ref to be retrieved via the factory.

  char *hostname_;
  // Hostname of server.

  CORBA::ULong portnum_;
  // default port number of server.

  u_int loop_count_;
  // Number of times to do the cube operations.

  int exit_later_;
  // Flag to tell server to not exit immediately

  Cubit_Factory_ptr factory_;
  // factory pointer for cubit.

  CORBA::Object_ptr objref_;
  // storage of the factory objref

  CORBA::Environment env_;
  // Environment variable

  Cubit_ptr cubit_;
  // Cubit obj ref

  CORBA::ORB_ptr orb_ptr_;
  // Remember our orb

  u_int call_count_;
  // # of calls made to functions

  u_int error_count_;
  // #  of errors incurred in the lifetime of the application.
};

#if defined (DEBUG)
#define dmsg1(X) { if (_debug_level > 0) ACE_DEBUG (X); }
#define dmsg2(X) { if (_debug_level > 1) ACE_DEBUG (X); }
#else
#define dmsg1(X) { }
#define dmsg2(X) { }
#endif /* DEBUG */

#endif /* _CLNT_H_ */
